/*
 * transaction.cc
 *
 * Copyright (C) 2009 Tim Marston <edam@waxworlds.org>
 *
 * This file is part of sqlite3cc (hereafter referred to as "this program").
 * See http://www.waxworlds.org/edam/software/sqlite3cc for more information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sqlite3cc/transaction.h>
#include <sqlite3cc/database.h>
#include <sqlite3cc/exception.h>
#include <boost/thread.hpp>
#include <boost/lexical_cast.hpp>


sqlite::basic_transaction::basic_transaction(
	database &database )
	:
	_database( database )
{
}


void sqlite::basic_transaction::begin()
{
	_database.exec( "BEGIN" );
}


void sqlite::basic_transaction::commit()
{
	_database.exec( "COMMIT" );
}


void sqlite::basic_transaction::rollback()
{
	_database.exec( "ROLLBACK" );
}


sqlite::immediate_transaction::immediate_transaction(
	database &database )
	:
	basic_transaction( database )
{
}


void sqlite::immediate_transaction::begin()
{
	_database.exec( "BEGIN IMMEDIATE" );
}


sqlite::exclusive_transaction::exclusive_transaction(
	database &database )
	:
	basic_transaction( database )
{
}


void sqlite::exclusive_transaction::begin()
{
	_database.exec( "BEGIN EXCLUSIVE" );
}


sqlite::recursive_transaction::recursive_transaction(
	database &database )
	:
	basic_transaction( database )
{
	static unsigned long long i = 0;
	unsigned long long my_i;
	static boost::mutex mutex;
	{
		boost::lock_guard< boost::mutex > lock( mutex );
		my_i = i++;
	}
	_sp_name = "_" + boost::lexical_cast< std::string >( my_i++ );
}


void sqlite::recursive_transaction::begin()
{
	_database.exec( "SAVEPOINT " + _sp_name );
}


void sqlite::recursive_transaction::commit()
{
	_database.exec( "RELEASE " + _sp_name );
}


void sqlite::recursive_transaction::rollback()
{
	_database.exec( "ROLLBACK TO " + _sp_name );

	// we have rolled back this transaction's savepoint, but the savepoint will
	// remain on the transaction stack unless we also release it
	_database.exec( "RELEASE " + _sp_name );
}
